@@ -1,4 +1,4 @@
-from logging import getLogger
+import logging
 from subprocess import PIPE, Popen, run
 from typing import Optional, Union
 
@@ -6,7 +6,7 @@ from rhoknp.processors.processor import Processor
 from rhoknp.processors.senter import RegexSenter
 from rhoknp.units import Document, Sentence
 
-logger = getLogger(__file__)
+logger = logging.getLogger(__name__)
 
 
 class Jumanpp(Processor):
@@ -61,9 +61,7 @@ class Jumanpp(Processor):
         if document.need_senter:
             logger.debug("document needs to be split into sentences")
             if self.senter is None:
-                logger.debug(
-                    "senter is not specified when initializing KNP: use RegexSenter with no option"
-                )
+                logger.debug("senter is not specified; use RegexSenter")
                 self.senter = RegexSenter()
             document = self.senter.apply_to_document(document)
 
