@@ -1,4 +1,4 @@
-from logging import getLogger
+import logging
 from subprocess import PIPE, Popen, run
 from typing import Optional, Union
 
@@ -7,7 +7,7 @@ from rhoknp.processors.processor import Processor
 from rhoknp.processors.senter import RegexSenter
 from rhoknp.units import Document, Sentence
 
-logger = getLogger(__file__)
+logger = logging.getLogger(__name__)
 
 
 class KNP(Processor):
@@ -69,23 +69,21 @@ class KNP(Processor):
         if document.need_senter:
             logger.debug("document needs to be split into sentences")
             if self.senter is None:
-                logger.debug(
-                    "senter is not specified when initializing KNP: use RegexSenter with no option"
-                )
+                logger.debug("senter is not specified; use RegexSenter")
                 self.senter = RegexSenter()
+                logger.debug(self.senter)
             document = self.senter.apply_to_document(document)
 
         if document.need_jumanpp:
             logger.debug("document needs to be processed by Juman++")
             if self.jumanpp is None:
-                logger.info(
-                    "jumanpp is not specified when initializing KNP: use Jumanpp with no option"
-                )
+                logger.info("jumanpp is not specified; use Jumanpp")
                 self.jumanpp = Jumanpp()
+                logger.debug(self.jumanpp)
             document = self.jumanpp.apply_to_document(document)
 
         with Popen(self.run_command, stdout=PIPE, stdin=PIPE, encoding="utf-8") as p:
-            knp_text, _ = p.communicate(input=document.to_jumanpp())
+            knp_text, _ = p.communicate(input=document.to_jumanpp() if document.need_knp else document.to_knp())
         return Document.from_knp(knp_text)
 
     def apply_to_sentence(self, sentence: Union[Sentence, str]) -> Sentence:
@@ -104,14 +102,12 @@ class KNP(Processor):
         if sentence.need_jumanpp:
             logger.debug("sentence needs to be processed by Juman++")
             if self.jumanpp is None:
-                logger.info(
-                    "jumanpp is not specified when initializing KNP: use Jumanpp with no option"
-                )
+                logger.info("jumanpp is not specified when initializing KNP: use Jumanpp with no option")
                 self.jumanpp = Jumanpp()
             sentence = self.jumanpp.apply_to_sentence(sentence)
 
         with Popen(self.run_command, stdout=PIPE, stdin=PIPE, encoding="utf-8") as p:
-            knp_text, _ = p.communicate(input=sentence.to_jumanpp())
+            knp_text, _ = p.communicate(input=sentence.to_jumanpp() if sentence.need_knp else sentence.to_knp())
         return Sentence.from_knp(knp_text)
 
     def is_available(self) -> bool:
