@@ -1,11 +1,11 @@
+import logging
 import re
-from logging import getLogger
 from typing import Union
 
 from rhoknp.processors.processor import Processor
 from rhoknp.units import Document, Sentence
 
-logger = getLogger(__file__)
+logger = logging.getLogger(__name__)
 
 
 class RegexSenter(Processor):
@@ -103,11 +103,7 @@ class RegexSenter(Processor):
     @staticmethod
     def _clean_up_candidates(sentence_candidates: list[str]) -> list[str]:
         """Remove empty sentence candidates."""
-        return [
-            sentence_candidate.strip()
-            for sentence_candidate in sentence_candidates
-            if sentence_candidate.strip()
-        ]
+        return [sentence_candidate.strip() for sentence_candidate in sentence_candidates if sentence_candidate.strip()]
 
     def __repr__(self) -> str:
         return f"{self.__class__.__name__}()"
