@@ -14,13 +14,11 @@ if TYPE_CHECKING:
     from rhoknp.units.sentence import Sentence
 
 
-@dataclass(frozen=True)
+@dataclass
 class MorphemeAttributes:
     """形態素の属性クラス．"""
 
-    JUMANPP_PAT = re.compile(
-        r"(?P<attrs>([^ ]+ [^ ]+ [^ ]+ [^ ]+ \d+ [^ ]+ \d+ [^ ]+ \d+ [^ ]+ \d+))"
-    )
+    JUMANPP_PAT = re.compile(r"(?P<attrs>([^ ]+ [^ ]+ [^ ]+ [^ ]+ \d+ [^ ]+ \d+ [^ ]+ \d+ [^ ]+ \d+))")
 
     surf: str  #: 表層表現．
     reading: str  #: 読み．
@@ -56,11 +54,9 @@ class Morpheme(Unit):
     """形態素クラス．"""
 
     JUMANPP_PAT: ClassVar[re.Pattern[str]] = re.compile(
-        (
-            rf"^({MorphemeAttributes.JUMANPP_PAT.pattern})"
-            + rf"(\s{Semantics.PAT.pattern})?"
-            + rf"(\s{Features.PAT.pattern})?$"
-        )
+        rf"^({MorphemeAttributes.JUMANPP_PAT.pattern})"
+        + rf"(\s{Semantics.PAT.pattern})?"
+        + rf"(\s{Features.PAT.pattern})?$"
     )
 
     count = 0
@@ -131,11 +127,7 @@ class Morpheme(Unit):
         Raises:
             AttributeError: 解析結果にアクセスできない場合．
         """
-        if self._sentence is not None:
-            return self._sentence
-        if self._base_phrase is not None:
-            return self.clause.sentence
-        raise AttributeError("sentence has not been set")
+        return self._sentence or self.base_phrase.sentence
 
     @sentence.setter
     def sentence(self, sentence: "Sentence") -> None:
@@ -153,7 +145,7 @@ class Morpheme(Unit):
         Raises:
             AttributeError: 解析結果にアクセスできない場合．
         """
-        return self.phrase.clause
+        return self.base_phrase.clause
 
     @property
     def phrase(self) -> "Phrase":
@@ -247,21 +239,27 @@ class Morpheme(Unit):
 
     @cached_property
     def span(self) -> tuple[int, int]:
-        """文中での文字レベルのスパン．"""
-        if self._sentence is None or self.index == 0:
+        """文における文字レベルのスパン．"""
+        if self.index == 0:
             start = 0
         else:
-            _, prev_end = self.sentence.morphemes[self.index - 1].span
-            start = prev_end
+            _, start = self.sentence.morphemes[self.index - 1].span
         end = start + len(self.text)  # TODO: correctly handle multibyte characters
         return start, end
 
+    @cached_property
+    def global_span(self) -> tuple[int, int]:
+        """文書全体における文字レベルのスパン．"""
+        offset = 0
+        for prev_sentence in self.document.sentences[: self.sentence.index]:
+            offset += len(prev_sentence.text)
+        start, end = self.span
+        return start + offset, end + offset
+
     @cached_property
     def children(self) -> list["Morpheme"]:
         """この形態素に係っている形態素のリスト．"""
-        return [
-            morpheme for morpheme in self.sentence.morphemes if morpheme.parent == self
-        ]
+        return [morpheme for morpheme in self.sentence.morphemes if morpheme.parent == self]
 
     @classmethod
     def from_jumanpp(cls, jumanpp_text: str) -> "Morpheme":
@@ -279,9 +277,7 @@ class Morpheme(Unit):
         return morpheme
 
     @classmethod
-    def _from_jumanpp_line(
-        cls, jumanpp_line: str, homograph: bool = False
-    ) -> "Morpheme":
+    def _from_jumanpp_line(cls, jumanpp_line: str, homograph: bool = False) -> "Morpheme":
         """形態素クラスのインスタンスを Juman++ の解析結果から初期化．
 
         Args:
